### Name: comp_train_pred
### Title: User-level functions for compressing parameters, training the
###   models with MCMC, and making predictions for test cases
### Aliases: begin.BPHO comp_train_pred cv_comp_train_pred
### Keywords: classif

### ** Examples

#####################################################################
########  These are demonstrations of using the whole package
#####################################################################

#####################################################################
########  Apply to Sequence Prediction Models
#####################################################################

## generate data from a hidden Markov model
data_hmm <- gen_hmm(n = 200, p = 10, no_h = 4, no_o = 2, 
                    prob_h_stay = 0.8, prob_o_stay = 0.8)

## compress parameters (transforming features in training data.)
compress (features = data_hmm$X[-(1:100),], is_sequence = 1, order = 4,
          ptn_file = ".ptn_file.log")

## print summary information of '.ptn_file.log'
display_ptn (ptn_file = ".ptn_file.log")

## draw 50 samples with slice sampling, saved in '.mc_file.log'
training (train_y = data_hmm$y[-(1:100)], no_cls = 2, 
          mc_file = ".mc_file.log", ptn_file = ".ptn_file.log", 
          iters_mc = 50)

## draw 50 more samples from the last iteration in '.mc_file.log'
training (train_y = data_hmm$y[-(1:100)], no_cls = 2, 
          mc_file = ".mc_file.log", ptn_file = ".ptn_file.log", 
          iters_mc = 50, start_over = FALSE)

## display general information of Markov chain stored in '.mc_file.log'
summary_mc (mc_file = ".mc_file.log")

## find medians of Markov chain samples of all betas
med_betas <- medians_betas (mc_file = ".mc_file.log", 
             iter_b = 50, forward = 1, samplesize = 50)

## take the id of group with highest absolute median
g_impt <- med_betas [1, "groupid"] 

## particularly read `betas' by specifying the group and class id
plot ( read_betas (ix_g = g_impt, ix_cls = 2, mc_file = ".mc_file.log", 
       iter_b = 50, forward = 1, samplesize = 50) )

## get information about pattern groups
display_ptn (".ptn_file.log", ix_g = g_impt )

## read Markov chain values of log-likelihood from  '.mc_file.log'
plot ( read_mc (group = "lprobs", ix = 1, mc_file = ".mc_file.log", 
       iter_b = 50, forward = 1, samplesize = 50) )

## predict for test cases
pred_probs <- predict_bpho (test_x = data_hmm$X[1:100,],
              mc_file = ".mc_file.log", ptn_file = ".ptn_file.log",
              iter_b = 50, forward = 1, samplesize = 50 )
              
## evaluate predictions with true value of response
evaluate_prediction (data_hmm$y[1:100], pred_probs)

#####################################################################
########  Apply to General Classification Models
#####################################################################

## generating a classification data
data_class <- gen_bin_ho(n = 200, p = 3, order = 3, alpha = 1,
              sigmas = c(0.3,0.2,0.1), nos_features = rep(4,3), beta0 = 0)

## compress parameters (transforming features in training data.)
compress (features = data_class$X[-(1:100),], is_sequence = 0, order = 20,
          ptn_file = ".ptn_file.log")

## print summary information of '.ptn_file.log'
display_ptn (ptn_file = ".ptn_file.log")

## draw 50 samples with slice sampling, saved in '.mc_file.log'
training (train_y = data_class$y[-(1:100)], no_cls = 2, 
          mc_file = ".mc_file.log", ptn_file = ".ptn_file.log", 
          iters_mc = 50)

## draw 50 more samples from the last iteration in '.mc_file.log'
training (train_y = data_class$y[-(1:100)], no_cls = 2, 
          mc_file = ".mc_file.log", ptn_file = ".ptn_file.log", 
          iters_mc = 50, start_over = FALSE)

## display general information of Markov chain stored in '.mc_file.log'
summary_mc (mc_file = ".mc_file.log")

## find medians of Markov chain samples of all betas
med_betas <- medians_betas (mc_file = ".mc_file.log", 
             iter_b = 50, forward = 1, samplesize = 50)

## take the id of group with highest absolute median
g_impt <- med_betas [1, "groupid"] 

## particularly read `betas' by specifying the group and class id
plot ( read_betas (ix_g = g_impt, ix_cls = 2, mc_file = ".mc_file.log", 
       iter_b = 50, forward = 1, samplesize = 50) )

## get information about pattern groups
display_ptn (".ptn_file.log", ix_g = g_impt )

## read Markov chain values of log-likelihood from  '.mc_file.log'
plot ( read_mc (group = "lprobs", ix = 1, mc_file = ".mc_file.log", 
       iter_b = 50, forward = 1, samplesize = 50) )

## predict for test cases
pred_probs <- predict_bpho (test_x = data_class$X[1:100,],
              mc_file = ".mc_file.log", ptn_file = ".ptn_file.log",
              iter_b = 50, forward = 1, samplesize = 50 )
              
## evaluate predictions with true value of response
evaluate_prediction (data_class$y[1:100], pred_probs)

#####################################################################
########  Demonstrations of using a single function 'comp_train_pred'
#####################################################################

## generating a classification data
data_class <- gen_bin_ho(n = 200, p = 3, order = 3, alpha = 1,
              sigmas = c(0.3,0.2,0.1), nos_features = rep(4,3), beta0 = 0)

## carry out compression, training, and prediction with a function 
comp_train_pred (
    ################## specify data information  #######################
    test_x = data_class$X[1:100,], train_x = data_class$X[-(1:100),],
    train_y = data_class$y[1:100], no_cls = 2, nos_features = rep(4,3),
    ################## specify for compression #########################
    is_sequence = 0, order = 3, ptn_file=".ptn.log", new_comp = 1, comp = 1,
    ################## specify for priors  #############################
    alpha = 1, log_sigma_widths = c(), log_sigma_modes = c(),
    ################## specify for mc sampling #########################
    mc_file = ".mc.log", iters_mc = 200, iters_bt = 10,
    iters_sgm = 50, w_bt = 5, m_bt = 20, w_sgm = 1, m_sgm = 20,
    ini_log_sigmas = c(), start_over = TRUE, 
    ################## specify for prediction ##########################
    pred_file = "pred_file.csv", iter_b = 100, forward = 2, samplesize = 50)

## evaluate predictions with true value of response
evaluate_prediction (data_class$y[1:100], read.csv("pred_file.csv") )

#####################################################################
#####  Demonstrations of using a single function 'cv_comp_train_pred'
#####################################################################

## generating a classification data
data_class <- gen_bin_ho(n = 200, p = 3, order = 3, alpha = 1,
              sigmas = c(0.3,0.2,0.1), nos_features = rep(4,3), beta0 = 0)

## carry out cross-validation with data set data_class
cv_comp_train_pred (
    ################## Specify data,order,no_fold ######################
    no_fold = 2, train_x = data_class$X,train_y = data_class$y,
    no_cls = 2, nos_features = rep(4,3),
    ################## specify for compression #########################
    is_sequence = 0, order = 3, ptn_file=".ptn.log",  comp = 1,
    ################## specify for priors  #############################
    alpha = 1,log_sigma_widths = c(),log_sigma_modes = c(),
    ################## specify for mc sampling #########################
    mc_file = ".mc.log", iters_mc = 200, iters_bt = 10,
    iters_sgm = 50,w_bt = 5,m_bt = 20,w_sgm = 1,m_sgm = 20,
    ini_log_sigmas = c(), 
    ################## specify for prediction ##########################
    pred_file = "pred_file.csv", iter_b = 100, forward = 2, samplesize = 50)




